#if defined(__XC)
#include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
#include <htc.h>        /* HiTech General Include File */
#endif

#include "user.h"
#include "RC5.h"

unsigned char ToggleBit = 0;
unsigned char SAMPLE_PERIOD = 0;
unsigned char HALF_PERIOD = 0;
unsigned char FAIL_PERIOD = 0;

void SendBit_RC5(unsigned char b)
{
	if (!!b)
		SendOne_RC5();
	else
		SendZero_RC5();
}

void SendZero_RC5(void)
{
	TMR0 = PER_889;
	INTCONbits.TMR0IF = 0;
	PWM1CONbits.PWM1EN = 1;
	while (!INTCONbits.TMR0IF);
	TMR0 = PER_889;
	INTCONbits.TMR0IF = 0;
	PWM1CONbits.PWM1EN = 0;
	while (!INTCONbits.TMR0IF);
}

void SendOne_RC5(void)
{
	TMR0 = PER_889;
	INTCONbits.TMR0IF = 0;
	PWM1CONbits.PWM1EN = 0;
	while (!INTCONbits.TMR0IF);
	TMR0 = PER_889;
	INTCONbits.TMR0IF = 0;
	PWM1CONbits.PWM1EN = 1;
	while (!INTCONbits.TMR0IF);
}

void WriteCommand_RC5(unsigned char device, unsigned char command)
{
	unsigned char i;
	// Make sure that line is in IDLE state
	PWM1CONbits.PWM1EN = 0;

	// 1. Send Start
	SendBit_RC5(1);
	SendBit_RC5(1);
	// 2. Change toggle bit
	ToggleBit ^= 1;
	SendBit_RC5(ToggleBit);
	// 3. SendAddress
	for (i = 0; i < 5; i++) {
		SendBit_RC5(device & 0x10);
		device <<= 1;
	}
	// 4. SendCommand
	for (i = 0; i < 6; i++) {
		SendBit_RC5(command & 0x20);
		command <<= 1;
	}

	// Put line in IDLE state
	PWM1CONbits.PWM1EN = 0;
}

void ReadCommand_RC5(void)
{
	unsigned int data = 0;
	unsigned char i;
	unsigned char tmr;

	// Wait for idle
	IOCAPbits.IOCAP2 = 1;
	IOCANbits.IOCAN2 = 1;
	IOCAFbits.IOCAF2 = 0;
	TMR0 = 56;	//3ms
	INTCONbits.T0IF = 0;
	while (!INTCONbits.T0IF);
	if (IOCAFbits.IOCAF2)
		return;

	// 1.Detect start bit
	IOCAPbits.IOCAP2 = 0;
	IOCANbits.IOCAN2 = 1;
	IOCAFbits.IOCAF2 = 0;
	while (!IOCAFbits.IOCAF2);

	// START bit detected
	IOCAPbits.IOCAP2 = 1;
	IOCANbits.IOCAN2 = 0;
	IOCAFbits.IOCAF2 = 0;
	TMR0 = 187; // 1.1ms
	INTCONbits.T0IF = 0;
	while (!IOCAFbits.IOCAF2 && !INTCONbits.T0IF);
	if (INTCONbits.T0IF)
		return;

	tmr = TMR0;
	HALF_PERIOD = (tmr - 187);
	SAMPLE_PERIOD = (HALF_PERIOD / 2) * 3;
	FAIL_PERIOD = SAMPLE_PERIOD + HALF_PERIOD;


	IOCAPbits.IOCAP2 = 0;
	IOCANbits.IOCAN2 = 1;
	IOCAFbits.IOCAF2 = 0;
	while (!IOCAFbits.IOCAF2);
	IOCAPbits.IOCAP2 = 1;
	IOCANbits.IOCAN2 = 1;


	for (i = 0; i < 12; i++) {
		data <<= 1;
		/* Start sync */
		TMR0 = 255 - SAMPLE_PERIOD + 1;
		INTCONbits.TMR0IF = 0;
		while (!INTCONbits.TMR0IF);
		data |= PORTAbits.RA2;

		/* Check for fault */
		TMR0 = 255 - FAIL_PERIOD + 1;
		INTCONbits.TMR0IF = 0;
		IOCAFbits.IOCAF2 = 0;
		while (!IOCAFbits.IOCAF2 && !INTCONbits.TMR0IF);
		if (INTCONbits.TMR0IF)
			return;

	}
	IS_READ = 0;
	VALUE = data;
}