/******************************************************************************/
/*Files to Include                                                            */
/******************************************************************************/

#if defined(__XC)
#include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
#include <htc.h>        /* HiTech General Include File */
#endif

#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */
#include "user.h"
#include "OLIMEX.h"
#include "RC5.h"
#include "SIRC.h"

/******************************************************************************/
/* Interrupt Routines                                                         */
/******************************************************************************/

/* Baseline devices don't have interrupts. Note that some PIC16's 
 * are baseline devices.  Unfortunately the baseline detection macro is 
 * _PIC12 */
#ifndef _PIC12

void interrupt isr(void)
{
	if (SSP1IF) {
		unsigned char status;
		unsigned char foo;
		int i;
		if (SSPSTAT & STOP_C) {
			switch (DATA_BUFFER[0]) {
			case GET_ID:
				DATA_OUT[0] = DEVICE_ID;
				break;
			case SET_MODE:
				if (DATA_BUFFER[1] == 0x00)
					MODE = RC5;
				else if (DATA_BUFFER[1] == 0x01)
					MODE = SIRC;
				break;
			case WRITE:
				if (MODE == RC5)
					WriteCommand_RC5(
						DATA_BUFFER[1],
						DATA_BUFFER[2]);
				else
					SendCommand_SIRC(
						DATA_BUFFER[1],
						DATA_BUFFER[2]);
				break;
			case READ:
				if (!IS_READ) {
					DATA_OUT[0] = (VALUE >> 8) & 0xFF;
					DATA_OUT[1] = VALUE & 0xFF;
				} else {
					DATA_OUT[0] = 0xFF;
					DATA_OUT[1] = 0xFF;
				}
				break;
			case SET_ADDRESS:
				SetAddress(DATA_BUFFER[1]);
				break;
			default:
				break;
			}
		} else {
			status = SSPSTAT & 0x2D; //DA | P | S | R/W | x | BF
			switch (status) {
				//STATE 1
			case STATE1:
			{
				foo = SSPBUF;
				DATA_INDEX = 0;
				for (i = 0; i < MAX_BUFFER_SIZE; i++)
					DATA_BUFFER[i] = 0;
				if (SEN)
					CKP = 1;
			}
				break;
				//STATE 2
			case STATE2:
			{
				DATA_BUFFER[DATA_INDEX++] = SSPBUF;
				if (DATA_INDEX >= MAX_BUFFER_SIZE)
					DATA_INDEX = 0;
				if (SEN)
					CKP = 1;
			}
				break;
				//STATE 3
			case STATE3:
			{
				foo = SSPBUF;
				while (BF);
				SSPBUF = DATA_OUT[0];
				if (SEN)
					CKP = 1;
				IS_READ = 1;
			}
				break;
				//STATE 4
			case STATE4:
			{
				foo = SSPBUF;
				while (BF);
				SSPBUF = DATA_OUT[1];
				if (SEN)
					CKP = 1;
			}
				break;
				//STATE 5
			case STATE5:
				break;
			default:
				break;
			}
		}
		SSP1IF = 0;
	}
}
#endif


